//----------------------------------------------------------------------
//	Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		MacStarListHandler.cpp
//
//	Contains:	List Box handler
//								
//	Authors:	Marko Kudjerski
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//
//----------------------------------------------------------------------

#include "MacStarListBoxHandler.h"
#include "Scope.h"

// items defined in Controller.cpp used for the list of stars
extern TListBox* gStarListBox;
extern UInt16    gListSelectionIndex;

//----------------------------------------------------------------------
//	TMacListBox::TMacListBox
//
//	Purpose:	Default TMacListBox constructor
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//
//----------------------------------------------------------------------
TMacListBox::TMacListBox(char* title = "", char* text = "")
{
	windowTitle.Assign(title);
	windowText.Assign(text);
}

//----------------------------------------------------------------------
//	TMacListBox::~TMacListBox
//
//	Purpose:	destruct 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//----------------------------------------------------------------------
TMacListBox::~TMacListBox(void)
{
}


//----------------------------------------------------------------------
//	TMacListBox::DisplayAndRunWarningBox()
//
//	Purpose:	Create and run the list dialog box procedure
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//----------------------------------------------------------------------
UInt32 TMacListBox::DisplayAndRunListBox(void)
{
	return this->CreateDialogBox();
}		


void TMacListBox::SetListSelectionIndex(void)
{
	WindowRef		dialogWindow = ::FrontWindow();
	
	ControlRef		listControl;
    ControlID 		listPopopMenuID;
	
    listPopopMenuID.signature = kPluginApplicationSignature;
    listPopopMenuID.id = kListBox;
	
	::GetControlByID(dialogWindow, &listPopopMenuID, &listControl);

	// Get the index of the currently selected item in the list
	gListSelectionIndex = ::GetControlValue(listControl) - 1;
}


//----------------------------------------------------------------------
//	TMacListBox::CreateDialogBox()
//
//	Purpose:	Create and show the list dialog box 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//----------------------------------------------------------------------
OSStatus TMacListBox::CreateDialogBox()
{
    IBNibRef 		nibRef;
    EventTypeSpec 	dialogSpec = {kEventClassCommand, kEventCommandProcess};
    WindowRef 		dialogWindow;
    EventHandlerUPP	dialogUPP;
    OSStatus		err = noErr;
	
	
	CFBundleRef	scopePluginBundleRef = ::CFBundleGetBundleWithIdentifier(kScopePluginBundleIdentifier);

    // Find the dialog nib
	err = ::CreateNibReferenceWithCFBundle(	scopePluginBundleRef, CFSTR("ListBox"), &nibRef);
	require_noerr( err, CantFindDialogNib );

    // Load the window inside it
    err = ::CreateWindowFromNib(nibRef, CFSTR("dialog"), &dialogWindow);
    require_noerr( err, CantCreateDialogWindow );

    // We don't need the nib reference anymore.
    ::DisposeNibReference(nibRef);
	
	//Set window title
	SetWindowTitleWithCFString (dialogWindow, CFStringCreateWithPascalString(NULL,StringPtr(windowTitle),NULL));	
	
	//Set window text
	ControlRef		windowTextRef;
    ControlID 		windowTextID;
    windowTextID.signature = kPluginApplicationSignature;
    windowTextID.id = kStaticText;
	err = ::GetControlByID(dialogWindow, &windowTextID, &windowTextRef);
    require_noerr( err, CantFindListControl);
	SetControlData(windowTextRef, kControlEntireControl, kControlStaticTextTextTag,windowText.Length(),windowText.TextPtr());
	
/*	ControlRef		windowOKRef;
    ControlID 		windowOKID;
    windowOKID.signature = kPluginApplicationSignature;
    windowOKID.id = kOKButtonID;
	err = ::GetControlByID(dialogWindow, &windowOKID, &windowOKRef);
    require_noerr( err, CantFindListControl);
	SetControlTitleWithCFString(windowOKRef,CFStringCreateWithPascalString(NULL,StringPtr(windowText),NULL));
*/

  	// Install our event handler
    dialogUPP =  ::NewEventHandlerUPP (MacStarListEventHandler);
    err = ::InstallWindowEventHandler (dialogWindow, dialogUPP, 1, &dialogSpec, (void *) dialogWindow, NULL);
    require_noerr( err, CantInstallDialogHandler );
	

	//Populate list
	ControlRef		listControl;
    ControlID 		listID;
	MenuRef starListPopupMenuRef;

    listID.signature = kPluginApplicationSignature;
    listID.id = kListBox;	
	err = ::GetControlByID(dialogWindow, &listID, &listControl);
    require_noerr( err, CantFindListControl);
	starListPopupMenuRef = ::GetControlPopupMenuHandle(listControl);
	::DeleteMenuItems(starListPopupMenuRef, 1, 3);

	for (UInt16 index = 0; (index < kBrightStarListSize) ; index++)
	{
		LStr255 star(starList[index].displayText);
		::AppendMenu(starListPopupMenuRef, "\pStar Name");
		::SetMenuItemText(starListPopupMenuRef, index+1, star);
	}
	
	// Set the control maximum to the number of stars used
	::SetControlMaximum(listControl, (kBrightStarListSize - 1) );

	// gListSelectionIndex is an array index, thus it starts at 0 but list index starts at 1
	::SetControlValue(listControl, gListSelectionIndex + 1);

    // Show the window
    ::ShowWindow( dialogWindow );
    
    // Run modally
    ::RunAppModalLoopForWindow(dialogWindow);
    
    ::HideWindow(dialogWindow);
    ::DisposeWindow(dialogWindow);
    ::DisposeEventHandlerUPP(dialogUPP);
	               
	CantFindDialogNib:
	CantCreateDialogWindow:
	CantInstallDialogHandler:
	CantFindListControl:
                
    return err;
}

//----------------------------------------------------------------------
//	MacStarListEventHandler()
//
//	Purpose:	Handle events in the list dialog box 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//----------------------------------------------------------------------
pascal OSStatus MacStarListEventHandler (	EventHandlerCallRef myHandler, 
											EventRef event, void * userData)
{
    OSStatus 		result = eventNotHandledErr;
    HICommand		command;
    pBoolean		stopModalLoop = false;

	WindowRef		dialogWindowRef = (WindowRef) userData;

	// Get the HI Command
    ::GetEventParameter (event, kEventParamDirectObject, typeHICommand, NULL, 
                            sizeof (HICommand), NULL, &command);
							
	gStarListBox->m_OK = false;

    // Look for our 'ok ' and 'not!' commands
    switch (command.commandID)
    {
        case 'ok  ':
				gStarListBox->m_OK = true;
				gStarListBox->SetListSelectionIndex();
				stopModalLoop = true;
                result = noErr;
                break;
        case 'not!':
                stopModalLoop = true;
                result = noErr;
                break;
		default:
				break;
    }
	   
    // Stop the modal loop.
    if (stopModalLoop)
        ::QuitAppModalLoopForWindow((WindowRef)userData);

    //Return how we handled the event.
	return result;
}  