
//----------------------------------------------------------------------
//	Copyright  1990-2002 by SPACE.com Canada, Inc., All rights reserved.
//
//	File:		ScopeConfigBox.h
//
//	Contains:	Telescope Configuration Dialog Box class.
//								
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/12/12	DWW			4.0.0		New internal telescope support for Pro 4
//
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//	Class:		TScopeConfigBox
//
//	Purpose:	Base class for the telescope configuration dialog box
//			
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/12/12	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
class TScopeConfigBox
{
	public:
	
	static TScopeConfigBox*	CreateNewConfigBox(void);
	static pBoolean			ReadPrefsFromFile(	scopeType&	outScopeTypeIndex,
												UInt32& 	outSerialPortIndex,
												pBoolean& 	outSetTimeAndLocOnOpen);
												
	static pBoolean			SavePrefsToFile(	scopeType 	inScopeTypeIndex,
												UInt32 		inSerialPortIndex, 
												pBoolean 	inSetTimeAndLocOnOpen);

	//construction
	public:			 
							TScopeConfigBox(void);
		virtual		 		~TScopeConfigBox(void);
		
	public:
		virtual pBoolean	SetGlobalsToSelectedScopeAndPort(void) = 0;
		virtual UInt32		DisplayAndRunConfigBox(void) = 0;
};

#if TARGET_OS_MAC
//----------------------------------------------------------------------
//	Class:		TMacScopeConfigBox
//
//	Purpose:	Macintosh class for the telescope configuration box
//			
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/12/12	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
class TMacScopeConfigBox : public TScopeConfigBox
{
	// construction
	public:			
						 TMacScopeConfigBox(void);
		virtual		 	~TMacScopeConfigBox(void);
		
	public:
		virtual UInt32		DisplayAndRunConfigBox(void);
		virtual pBoolean 	SetGlobalsToSelectedScopeAndPort();
		
	protected:
		OSStatus 		CreateDialogBox();
		void 			PopulatePortListBox(ControlRef inPortPopupMenuControlRef);
		void 			PopulateScopesListBox(ControlRef inScopePopupMenuControlRef);
};
#endif /*TARGET_OS_MAC*/

#if TARGET_OS_WIN32
//----------------------------------------------------------------------
//	Class:		TWinScopeConfigBox
//
//	Purpose:	Windows class for the telescope configuration box
//			
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/12/12	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
class TWinScopeConfigBox : public TScopeConfigBox
{
	// Construction
	public:			
						 TWinScopeConfigBox(HINSTANCE 	inPluginDLLInstanceHandle);
		virtual		 	~TWinScopeConfigBox(void);
	
	public:
		virtual UInt32		DisplayAndRunConfigBox(void);
		virtual pBoolean	SetGlobalsToSelectedScopeAndPort();
	protected:
		void 			CreateDialogBox();
		void			PopulatePortListBox(HWND inSerialPortComboBoxHandle);
		void			PopulateScopesListBox(HWND inScopeComboBoxHandle);
		
	private:
		HINSTANCE 		pluginDLLInstanceHandle;
};
#endif /*TARGET_OS_WIN32*/