//----------------------------------------------------------------------
//	Copyright  1990-2002 by SPACE.com Canada, Inc., All rights reserved.
//
//	File:		MacOrionAlignmentHandler.cpp
//
//	Contains:	Telescope and serial port configuration dialog handler
//								
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/12/12	DWW			4.0.0		New internal telescope support for Pro 4
//
//----------------------------------------------------------------------

#include "MacOrionAlignmentHandler.h"


//----------------------------------------------------------------------
//	TMacScopeWarningBox::TMacScopeWarningBox
//
//	Purpose:	Default TMacScopeWarningBox constructor
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//
//----------------------------------------------------------------------
TMacScopeWarningBox::TMacScopeWarningBox(void)
{
}

//----------------------------------------------------------------------
//	TMacScopeWarningBox::~TMacScopeWarningBox
//
//	Purpose:	destruct 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//----------------------------------------------------------------------
TMacScopeWarningBox::~TMacScopeWarningBox(void)
{
}


//----------------------------------------------------------------------
//	TMacScopeWarningBox::DisplayAndRunWarningBox()
//
//	Purpose:	Create and run the configure dialog box procedure
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//----------------------------------------------------------------------
UInt32 TMacScopeWarningBox::DisplayAndRunWarningBox(void)
{
	return this->CreateDialogBox();
}		

//----------------------------------------------------------------------
//	TMacScopeWarningBox::CreateDialogBox()
//
//	Purpose:	Create and show the configure dialog box 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
OSStatus TMacScopeWarningBox::CreateDialogBox()
{
    IBNibRef 		nibRef;
    EventTypeSpec 	dialogSpec = {kEventClassCommand, kEventCommandProcess};
    WindowRef 		dialogWindow;
    EventHandlerUPP	dialogUPP;
    OSStatus		err = noErr;
	
	CFBundleRef	scopePluginBundleRef = ::CFBundleGetBundleWithIdentifier(kScopePluginBundleIdentifier);

    // Find the dialog nib
	err = ::CreateNibReferenceWithCFBundle(	scopePluginBundleRef, CFSTR("WarningDialog"), &nibRef);
	require_noerr( err, CantFindDialogNib );

    // Load the window inside it
    err = ::CreateWindowFromNib(nibRef, CFSTR("dialog"), &dialogWindow);
    require_noerr( err, CantCreateDialogWindow );

    // We don't need the nib reference anymore.
    ::DisposeNibReference(nibRef);

  	// Install our event handler
    dialogUPP =  ::NewEventHandlerUPP (MacWarningBoxEventHandler);
    err = ::InstallWindowEventHandler (dialogWindow, dialogUPP, 1, &dialogSpec, (void *) dialogWindow, NULL);
    require_noerr( err, CantInstallDialogHandler );
	
    // Show the window
    ::ShowWindow( dialogWindow );
    
    // Run modally
    ::RunAppModalLoopForWindow(dialogWindow);
    
    ::HideWindow(dialogWindow);
    ::DisposeWindow(dialogWindow);
    ::DisposeEventHandlerUPP(dialogUPP);
	               
	CantFindDialogNib:
	CantCreateDialogWindow:
	CantInstallDialogHandler:
                
    return err;
}

//----------------------------------------------------------------------
//	MacWarningBoxEventHandler()
//
//	Purpose:	Handle events in the configure dialog box 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Orion alignment dialogs
//----------------------------------------------------------------------
pascal OSStatus MacWarningBoxEventHandler (	EventHandlerCallRef myHandler, 
											EventRef event, void * userData)
{
    OSStatus 		result = eventNotHandledErr;
    HICommand		command;
    pBoolean		stopModalLoop = false;

	//ControlRef		setTimeAndLocRadioControl; 
	//ControlID 		setTimeAndLocRadioControlID; // removed until further notice

	WindowRef		dialogWindowRef = (WindowRef) userData;

	// Find our set time and location checkbox control
    // setTimeAndLocRadioControlID.signature = kPluginApplicationSignature;
    // setTimeAndLocRadioControlID.id = kSetTimeLocRadioID;

	// Get the HI Command
    ::GetEventParameter (event, kEventParamDirectObject, typeHICommand, NULL, 
                            sizeof (HICommand), NULL, &command);
							
    // Look for our 'ok ' and 'not!' commands
    switch (command.commandID)
    {
        case 'ok  ':
				fWarningBox->m_OKed = true;
				stopModalLoop = true;
                result = noErr;
                break;
        case 'not!':
				fWarningBox->m_OKed = false;
                stopModalLoop = true;
                result = noErr;
                break;
		default:
				break;
    }
	   
    // Stop the modal loop.
    if (stopModalLoop)
        ::QuitAppModalLoopForWindow((WindowRef)userData);

    //Return how we handled the event.
    return result;
}