/*
 *  WinConfigDialogHandler.cpp
 *  ScopeConfigUtility
 *
 *  Created by dwhipps on Wed Feb 20 2002.
 *  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
 *
 */
 
// Some globals for use throughout
extern TScope*							gScope;
extern TScopeConfigBox*					gScopeConfigureBox;
extern TSerialCommunicator*				gSerialCommunicator;
extern GSXSerialPortNameRecordHandle 	gPortNameRecordHandle;
extern UInt16							gNumberOfReadySerialPorts;
extern UInt32 							gSelectedSerialPortIndex;
extern scopeType						gSelectedScopeType;
extern GSXSerialPortNameRecord 			gSelectedPortNameRecord;
extern GSSerialPortInfoRecord 			gDefaultPortInfoRecord;
extern pBoolean							gCanSetScopeTimeAndLocation;
extern pBoolean							gShouldSetScopeTimeAndLocationOnConnect;

#pragma mark ---TWinScopeConfigBox---
//----------------------------------------------------------------------
//	TWinScopeConfigBox::TWinScopeConfigBox
//
//	Purpose:	Default TWinScopeConfigBox constructor
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TWinScopeConfigBox::TWinScopeConfigBox(HINSTANCE inPluginDLLInstanceHandle)
{
	pluginDLLInstanceHandle = inPluginDLLInstanceHandle;
}

//----------------------------------------------------------------------
//	TWinScopeConfigBox::~TWinScopeConfigBox
//
//	Purpose:	destruct 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TWinScopeConfigBox::~TWinScopeConfigBox(void)
{
}

//----------------------------------------------------------------------
//	TWinScopeConfigBox::DisplayAndRunConfigBox()
//
//	Purpose:	destruct 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
UInt32 TWinScopeConfigBox::DisplayAndRunConfigBox(void)
{
		return this->CreateDialogBox();
}

//----------------------------------------------------------------------
//	TWinScopeConfigBox::CreateDialogBox()
//
//	Purpose:	Create and show the configure dialog box 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TWinScopeConfigBox::CreateDialogBox()
{
	DialogBox(pluginDLLInstanceHandle, "ConfigureBox", (HWND)NULL, Win32DialogBoxEventHandler);
}

//----------------------------------------------------------------------
//	TWinScopeConfigBox::PopulateScopesListBox()
//
//	Purpose:	Populate the scopes list box with all available scopes 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TWinScopeConfigBox::PopulateScopesListBox(HWND inScopeComboBoxHandle)
{
	SInt32 index = 0;
	
	for(SInt16 index = 0; index < kNumberOfScopeTypesSupported; index++)
		::SendMessage(	scopeComboBoxHandle, CB_ADDSTRING, 0, (long) (kScopeNameArrary[index]).ConstTextPtr() );
}

//----------------------------------------------------------------------
//	TWinScopeConfigBox::PopulatePortListBox()
//
//	Purpose:	Populate the ports list box with all available ports 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TWinScopeConfigBox::PopulatePortListBox(HWND inSerialPortComboBoxHandle)
{
	SInt32 index = 0;

	for(index = 0; index < gNumberOfReadySerialPorts; index++)
		::SendMessage(	inSerialPortComboBoxHandle, CB_ADDSTRING, 0,
						(long) &(((*gPortNameRecordHandle)[iSerialPortArrayIndex]).userPortName[0]) );
}

//----------------------------------------------------------------------
//	TWinScopeConfigBox::SetGlobalsToSelectedScopeAndPort()
//
//	Purpose:	Set set the globals to the selected scope and port 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
UInt32 TWinScopeConfigBox::SetGlobalsToSelectedScopeAndPort(void)
{
	HWND dialogBoxHandle			= //windows method for finding the handle to a window...

	HWND serialPortComboBoxHandle	= GetDlgItem(dialogBoxHandle, IDD_COMMLIST);
	HWND scopeComboBoxHandle 		= GetDlgItem(dialogBoxHandle, IDD_SCOPELIST);
	HWND timeAndLocChkBoxHandle 	= GetDlgItem(dialogBoxHandle, IDD_TIMELOCBOX);
	
	gScopeType					= (scopeType) ::SendMessage(scopeComboBoxHandle, CB_GETCURSEL, 0, 0);
	gSerialPortIndex			= ::SendMessage(serialPortComboBoxHandle, CB_GETCURSEL, 0, 0);
	gShouldSetScopeTimeAndLocationOnConnect	= ::SendMessage(timeAndLocChkBoxHandle, BM_GETCHECK, 0, 0);
}

//----------------------------------------------------------------------
//	Win32ConfigureBoxDialogProc()
//
//	Purpose:	The Windows dialog box control procedure 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
BOOL CALLBACK Win32DialogBoxEventHandler(	HWND inDialogBoxHandle, 
											UINT inMessage,
											WPARAM wParam,
											LPARAM lParam)
{
	HWND serialPortComboBoxHandle	= GetDlgItem(inDialogBoxHandle, IDD_COMMLIST);
	HWND scopeComboBoxHandle 		= GetDlgItem(inDialogBoxHandle, IDD_SCOPELIST);
	HWND timeAndLocChkBoxHandle 	= GetDlgItem(inDialogBoxHandle, IDD_TIMELOCBOX);
	
	switch (inMessage)
	  {
	  case WM_INITDIALOG :
	  		// populate the comm and scope listboxes here
			gScopeConfigureBox->PopulatePortListBox(serialPortComboBoxHandle);
			gScopeConfigureBox->PopulateScopesListBox(scopeComboBoxHandle);
			
	  		// Set the focus to the currently saved comm port
	  		// and the default scope selection to the saved scope type
	  		::SendMessage(scopeComboBoxHandle,   CB_SETCURSEL, gSerialPortIndex, 0);
	  		::SendMessage(serPortComboBoxHandle, CB_SETCURSEL, gScopeType, 0);
	       	
	       	// Set the state of the time/loc checkbox. Grey if not available.
	       	::SendMessage(timeAndLocChkBoxHandle, BM_SETCHECK, gScopeCanSetLocation, 0);

	       	return TRUE ;

	  case WM_COMMAND :
	       switch (LOWORD (wParam))
	            {                    
	            case IDOK :
					// Set the globals to the selected scope and port 
					gScopeConfigureBox->SetGlobalsToSelectedScopeAndPort();
					
					 //SAVE THESE OUT TO THE PREFS FILE
					 pBoolean saveSuccess = TScopeConfigBox::SavePrefsToFile(gScopeType, gSerialPortIndex, gSetScopeTimeAndLocOnOpen);
					 assert(saveSuccess);
					 					 
	            	 EndDialog (inDialogBoxHandle, TRUE) ;
		             return TRUE ;
		             
	            case IDCANCEL :
	                 EndDialog (inDialogBoxHandle, FALSE) ;
		             return TRUE ;
	            }
	       break ;
	  }
	return FALSE ;
}
