//----------------------------------------------------------------------
//	Copyright  1990-2002 by SPACE.com Canada, Inc., All rights reserved.
//
//	File:		MacAdvancedDialogHandler.cpp
//
//	Contains:	Advanced telescope controls (testing only!)
//								
//	Authors:	Marko Kudjerski (based on configdialoghandler by Dave Whipps)
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	04/07/14	MK			5.0			Mostly used for testing of scope controls
//
//----------------------------------------------------------------------

#include "MacAdvancedDialogHandler.h"

#include "Scope.h"
		
// Some external globals used throughout note that these are defined in Controller.cpp and imported here.
extern TScope*							gScope;
extern TScopeAdvancedBox*				gScopeAdvancedBox;
extern TSerialCommunicator*				gSerialCommunicator;
extern GSXSerialPortNameRecordHandle 	gPortNameRecordHandle;
extern UInt16							gNumberOfReadySerialPorts;
extern UInt32 							gSelectedSerialPortIndex;
extern scopeType						gSelectedScopeType;
extern GSXSerialPortNameRecord 			gSelectedPortNameRecord;
extern GSSerialPortInfoRecord 			gDefaultPortInfoRecord;
extern pBoolean							gCanSetScopeTimeAndLocation;
//extern SInt32 						gSetTimeLocationAction;
ControlActionUPP slewRateScrollerUPP;
ControlID slewRateScrollerID;
ControlRef slewRateScroller;

//----------------------------------------------------------------------
//	TMacScopeAdvancedBox::TMacScopeAdvancedBox
//
//	Purpose:	Default TMacScopeAdvancedBox constructor
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TMacScopeAdvancedBox::TMacScopeAdvancedBox(void)
{
}

//----------------------------------------------------------------------
//	TMacScopeAdvancedBox::~TMacScopeAdvancedBox
//
//	Purpose:	destruct 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TMacScopeAdvancedBox::~TMacScopeAdvancedBox(void)
{
}


//----------------------------------------------------------------------
//	TMacScopeAdvancedBox::DisplayAndRunAdvancedBox()
//
//	Purpose:	Create and run the configure dialog box procedure
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
UInt32 TMacScopeAdvancedBox::DisplayAndRunAdvancedBox()
{
	return this->CreateDialogBox();
}		

//----------------------------------------------------------------------
//	TMacScopeAdvancedBox::CreateDialogBox()
//
//	Purpose:	Create and show the configure dialog box 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
OSStatus TMacScopeAdvancedBox::CreateDialogBox()
{
    IBNibRef 		nibRef;
    EventTypeSpec 	dialogSpec = {kEventClassCommand, kEventCommandProcess};
    WindowRef 		dialogWindow;
    EventHandlerUPP	dialogUPP;
    OSStatus		err = noErr;
	CFBundleRef	scopePluginBundleRef = ::CFBundleGetBundleWithIdentifier(kScopePluginBundleIdentifier);

    // Find the dialog nib
	err = ::CreateNibReferenceWithCFBundle(	scopePluginBundleRef, CFSTR("AdvancedDialog"), &nibRef);
	require_noerr( err, CantFindDialogNib );

    // Load the window inside it
    err = ::CreateWindowFromNib(nibRef, CFSTR("controls"), &dialogWindow);
    require_noerr( err, CantCreateDialogWindow );

    // We don't need the nib reference anymore.
    ::DisposeNibReference(nibRef);
	
	// Install our event handler
    dialogUPP =  ::NewEventHandlerUPP (MacAdvancedDialogBoxEventHandler);
    err = ::InstallWindowEventHandler (dialogWindow, dialogUPP, 1, &dialogSpec, (void *) dialogWindow, NULL);
    require_noerr( err, CantInstallDialogHandler );
	
	slewRateScrollerID.signature = kPluginApplicationSignature;
    slewRateScrollerID.id = kScopeSLEWRATE;

	::GetControlByID(dialogWindow, &slewRateScrollerID, &slewRateScroller);

    slewRateScrollerUPP = NewControlActionUPP (SlewRateLiveScroll);

	SetControlAction (slewRateScroller,slewRateScrollerUPP); 

    // Show the window
    ::ShowWindow( dialogWindow );
    
    // Run modally
    ::RunAppModalLoopForWindow(dialogWindow);
    
    ::HideWindow(dialogWindow);
    ::DisposeWindow(dialogWindow);
    ::DisposeEventHandlerUPP(dialogUPP);
	               
	CantFindDialogNib:
	CantCreateDialogWindow:
	CantInstallDialogHandler:
                
    return err;
}

static pascal void SlewRateLiveScroll (ControlHandle control, SInt16 part) {
    SInt16 currentValue, min, max, delta;
    currentValue = GetControl32BitValue (control); 
    min = GetControl32BitMinimum (control);
    max = GetControl32BitMaximum (control);
    delta = 0; 

    switch (part) 
    {
        case kControlUpButtonPart: 
            if (currentValue >min)
				delta = (currentValue-min < 5 ? -(currentValue-min): -5);
			break;
        case kControlDownButtonPart:
            if (currentValue < max)
				delta =  (max - currentValue > 5 ? 5 : max - currentValue);
            break;
	}
    if (delta != 0)
    {
        SetControl32BitValue (control, currentValue + delta); 

        if (control == slewRateScroller) {;}
//                picOffset.v -= delta;
    }
    else if (part == kControlIndicatorPart) 
    {
		if (control == slewRateScroller) {
			delta =  (max - currentValue > 5 ? 5 : max - currentValue);
		}
		else {
			delta =  (max - currentValue > 5 ? 5 : max - currentValue);
		}
//			picOffset.h = -(currentValue - min);
    }
}

//----------------------------------------------------------------------
//	MacDialogBoxEventHandler()
//
//	Purpose:	Handle events in the configure dialog box 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
pascal OSStatus MacAdvancedDialogBoxEventHandler (	EventHandlerCallRef myHandler, 
											EventRef event, void * userData)
{
    OSStatus 		result = eventNotHandledErr;
	HICommand		command;
    pBoolean		stopModalLoop = false;
	PScopeMotion	nudgeDirection;

	//WindowRef		dialogWindowRef = (WindowRef) userData;

	// Get the HI Command
    ::GetEventParameter (event, kEventParamDirectObject, typeHICommand, NULL, sizeof (HICommand), NULL, &command);

    switch (command.commandID)
    {
		case 'up  ':
				nudgeDirection.fMotionDirection = direction_UP;
				nudgeDirection.fRelativeSlewRate = rate_4;
//				gScope->SlewScope(TScope::UP,TScope::FOURTH);
                result = noErr;
                break;
        case 'down':
				nudgeDirection.fMotionDirection = direction_DOWN;
				nudgeDirection.fRelativeSlewRate = rate_4;
//				gScope->SlewScope(TScope::DOWN,TScope::FOURTH);
				result = noErr;
				break;
		case 'left':
				nudgeDirection.fMotionDirection = direction_LEFT;
				nudgeDirection.fRelativeSlewRate = rate_4;
//				gScope->SlewScope(TScope::LEFT,TScope::FOURTH);
                result = noErr;
                break;
		case 'rght':
				nudgeDirection.fMotionDirection = direction_RIGHT;
				nudgeDirection.fRelativeSlewRate = rate_4;
//				gScope->SlewScope(TScope::RIGHT,TScope::FOURTH);
                result = noErr;
                break;
		case 'stop':
				nudgeDirection.fMotionDirection = direction_NONE_STOP;
				nudgeDirection.fRelativeSlewRate = rate_3;
//				gScope->SlewScope(TScope::NONE,TScope::THIRD);
                result = noErr;
                break;
		default:
				break;
    }
	
	gScope->SlewScope(&nudgeDirection);
	
	// Stop the modal loop.
    if (stopModalLoop)
        ::QuitAppModalLoopForWindow((WindowRef)userData);

    //Return how we handled the event.
    return result;
}

