//----------------------------------------------------------------------
//	Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		MacConfigDialogHandler.cpp
//
//	Contains:	Telescope and serial port configuration dialog handler
//								
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/12/12	DWW			4.0.0		New internal telescope support for Pro 4
//
//----------------------------------------------------------------------

#include "MacConfigDialogHandler.h"

#include "Scope.h"

// Some external globals used throughout note that these are defined in Controller.cpp and imported here.
extern TScope*							gScope;
extern TScopeConfigBox*					gScopeConfigureBox;
extern TSerialCommunicator*				gSerialCommunicator;
extern GSXSerialPortNameRecordHandle 	gPortNameRecordHandle;
extern UInt16							gNumberOfReadySerialPorts;
extern UInt32 							gSelectedSerialPortIndex;
extern scopeType						gSelectedScopeType;
extern GSXSerialPortNameRecord 			gSelectedPortNameRecord;
extern GSSerialPortInfoRecord 			gDefaultPortInfoRecord;
extern pBoolean							gCanSetScopeTimeAndLocation;
//extern SInt32 						gSetTimeLocationAction;

//----------------------------------------------------------------------
//	TMacScopeConfigBox::TMacScopeConfigBox
//
//	Purpose:	Default TMacScopeConfigBox constructor
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------
TMacScopeConfigBox::TMacScopeConfigBox(void)
{
}

//----------------------------------------------------------------------
//	TMacScopeConfigBox::~TMacScopeConfigBox
//
//	Purpose:	destruct 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
TMacScopeConfigBox::~TMacScopeConfigBox(void)
{
}


//----------------------------------------------------------------------
//	TMacScopeConfigBox::DisplayAndRunConfigBox()
//
//	Purpose:	Create and run the configure dialog box procedure
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
UInt32 TMacScopeConfigBox::DisplayAndRunConfigBox(void)
{
	return this->CreateDialogBox();
}		

//----------------------------------------------------------------------
//	TMacScopeConfigBox::CreateDialogBox()
//
//	Purpose:	Create and show the configure dialog box 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
OSStatus TMacScopeConfigBox::CreateDialogBox()
{
    IBNibRef 		nibRef;
    EventTypeSpec 	dialogSpec = {kEventClassCommand, kEventCommandProcess};
    WindowRef 		dialogWindow;
    EventHandlerUPP	dialogUPP;
    OSStatus		err = noErr;
	
	//ControlRef		setTimeAndLocRadioControl; 
	//ControlID 		setTimeAndLocRadioControlID; // removed until further notice
		
	ControlRef		scopeListControl, portListControl;
    ControlID 		scopeListPopopMenuID, portListPopopMenuID;
	
	CFBundleRef	scopePluginBundleRef = ::CFBundleGetBundleWithIdentifier(kScopePluginBundleIdentifier);

    // Find the dialog nib
	err = ::CreateNibReferenceWithCFBundle(	scopePluginBundleRef, CFSTR("ConfigDialog"), &nibRef);
	require_noerr( err, CantFindDialogNib );

    // Load the window inside it
    err = ::CreateWindowFromNib(nibRef, CFSTR("dialog"), &dialogWindow);
    require_noerr( err, CantCreateDialogWindow );

    // We don't need the nib reference anymore.
    ::DisposeNibReference(nibRef);
	
	/*
	// Find our set time and location checkbox control
    setTimeAndLocRadioControlID.signature = kPluginApplicationSignature;
    setTimeAndLocRadioControlID.id = kSetTimeLocRadioID;
	*/

	// Find our port list popup control
    portListPopopMenuID.signature = kPluginApplicationSignature;
    portListPopopMenuID.id = kCommPortPopupMenuID;
    
	// Find our scope list popup control
    scopeListPopopMenuID.signature = kPluginApplicationSignature;
    scopeListPopopMenuID.id = kScopeTypePopupMenuID;
	
	err = ::GetControlByID(dialogWindow, &portListPopopMenuID, &portListControl);
    require_noerr( err, CantFindListControl);

	err = ::GetControlByID(dialogWindow, &scopeListPopopMenuID, &scopeListControl);
	require_noerr( err, CantFindListControl);

	//err = ::GetControlByID(dialogWindow, &setTimeAndLocRadioControlID, &setTimeAndLocRadioControl);
    //require_noerr( err, CantFindListControl);
    
	// Install our event handler
    dialogUPP =  ::NewEventHandlerUPP (MacDialogBoxEventHandler);
    err = ::InstallWindowEventHandler (dialogWindow, dialogUPP, 1, &dialogSpec, (void *) dialogWindow, NULL);
    require_noerr( err, CantInstallDialogHandler );
	
	// Populate the port and scope type popup menus
	this->PopulatePortListBox(portListControl);
	this->PopulateScopesListBox(scopeListControl);
	
	// Set to the currently selected serial port and scope type
	if (gSelectedSerialPortIndex == 0) // true only the first time through
	{
		::SetControlValue(portListControl, 1);
		::SetControlValue(scopeListControl, 1);
		//::SetControl32BitValue(setTimeAndLocRadioControl, 3);
	}	
	else
	{
		::SetControlValue(portListControl, gSelectedSerialPortIndex);
		::SetControlValue(scopeListControl, gSelectedScopeType);
		//::SetControl32BitValue(setTimeAndLocRadioControl, gSetTimeLocationAction);
	}
	
    // Show the window
    ::ShowWindow( dialogWindow );
    
    // Run modally
    ::RunAppModalLoopForWindow(dialogWindow);
    
    ::HideWindow(dialogWindow);
    ::DisposeWindow(dialogWindow);
    ::DisposeEventHandlerUPP(dialogUPP);
	               
CantFindDialogNib:
CantCreateDialogWindow:
CantInstallDialogHandler:
CantFindListControl:
                
    return err;
}


//----------------------------------------------------------------------
//	TMacScopeConfigBox::PopulatePortListBox()
//
//	Purpose:	Populate the ports list box with all available ports 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
void TMacScopeConfigBox::PopulatePortListBox(ControlRef inPortPopupMenuControlRef)
{
	GSXSerialPortNameRecord currentPortNameRecord = {0,0,0};	
	gNumberOfReadySerialPorts = TSerialCommunicator::FindReadySerialPorts(&gPortNameRecordHandle);
	
	// Get the popup menu handle
	MenuRef portListPopupMenuRef;
	portListPopupMenuRef = ::GetControlPopupMenuHandle(inPortPopupMenuControlRef);

	// Clear the default item
	::DeleteMenuItems(portListPopupMenuRef, 1, 1);
	
	if (gNumberOfReadySerialPorts > 0)
	{
		for (int index = 1; (index <= gNumberOfReadySerialPorts) && (index < 31) ; index++)
		{
			currentPortNameRecord = (*gPortNameRecordHandle)[index-1];
			AppendMenu(portListPopupMenuRef, (const unsigned char *) &(currentPortNameRecord.userPortName));
		}
		
		::SetControlMaximum(inPortPopupMenuControlRef, gNumberOfReadySerialPorts);
	}
	else
	{
		::CFStringRef noReadyPortsMessage = CFSTR("No ready serial ports");
		AppendMenu(portListPopupMenuRef, CFStringGetPascalStringPtr(noReadyPortsMessage, kCFStringEncodingMacRoman));
		::CFRelease(noReadyPortsMessage);
		
		::SetControlMaximum(inPortPopupMenuControlRef, 1);
		::SetControlValue(inPortPopupMenuControlRef, 1);
	}
}

//----------------------------------------------------------------------
//	TMacScopeConfigBox::PopulateScopesListBox()
//
//	Purpose:	Populate the scopes list box with all available scopes 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
void TMacScopeConfigBox::PopulateScopesListBox(ControlRef inScopePopupMenuControlRef)
{
	// Get the popup menu handles
	MenuRef scopeListPopupMenuRef;
	scopeListPopupMenuRef = ::GetControlPopupMenuHandle(inScopePopupMenuControlRef);

	// Clear the default item
	::DeleteMenuItems(scopeListPopupMenuRef, 1, 1);

	// Start at index = 1 since index = 0 is reserved for scope_NONE
	for (UInt16 index = 1; (index < kNumberOfScopeTypesSupported) && (index < 31) ; index++)
	{
		::AppendMenu(scopeListPopupMenuRef, "\pScope Name");
		::SetMenuItemText(scopeListPopupMenuRef, index, kScopeNameArrary[index]);
	}
	
	// Set the control maximum to the number of supported scopes (-1 because of the scope_NONE type)
	::SetControlMaximum(inScopePopupMenuControlRef, (kNumberOfScopeTypesSupported - 1) );
}

//----------------------------------------------------------------------
//	TMacScopeConfigBox::SetGlobalsToSelectedScopeAndPort()
//
//	Purpose:	Set set the globals to the selected scope and port 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
pBoolean TMacScopeConfigBox::SetGlobalsToSelectedScopeAndPort(void)
{
	pBoolean success = false;
	
	WindowRef		dialogWindow = ::FrontWindow();

	//ControlRef		setTimeAndLocRadioControl; 
	//ControlID 		setTimeAndLocRadioControlID; // removed until further notice
	
	ControlRef		scopeListControl, portListControl;
    ControlID 		scopeListPopopMenuID, portListPopopMenuID;
	
	// Find our set time and location checkbox control
    // setTimeAndLocRadioControlID.signature = kPluginApplicationSignature;
    // setTimeAndLocRadioControlID.id = kSetTimeLocRadioID;

	// Find our port list popup control
    portListPopopMenuID.signature = kPluginApplicationSignature;
    portListPopopMenuID.id = kCommPortPopupMenuID;
    
	// Find our scope list popup control
    scopeListPopopMenuID.signature = kPluginApplicationSignature;
    scopeListPopopMenuID.id = kScopeTypePopupMenuID;
	
	//::GetControlByID(dialogWindow, &setTimeAndLocRadioControlID, &setTimeAndLocRadioControl);
	::GetControlByID(dialogWindow, &portListPopopMenuID, &portListControl);
	::GetControlByID(dialogWindow, &scopeListPopopMenuID, &scopeListControl);
	
	// Get the index of the currently selected port and scope
	gSelectedSerialPortIndex = ::GetControlValue(portListControl);
	gSelectedScopeType 		 = (scopeType) ::GetControlValue(scopeListControl);	

	// Get the check state of the set scope time and location checkbox
	//SInt32 currentRadioValue = (SInt32) ::GetControl32BitValue(setTimeAndLocRadioControl);
	//gSetTimeLocationAction = currentRadioValue;

	// create the serial communicator here and 	init the serial comm
	// NOTE that the index of the gSelectedSerialPortIndex and the gSelectedSerialPortIndex are
	// offset by one (the menu items start at 1 not 0) so subtract one to get the correct index
	
	if (gPortNameRecordHandle)
	{
		gSelectedPortNameRecord = (*gPortNameRecordHandle)[gSelectedSerialPortIndex - 1];
		
		if (gSerialCommunicator != NULL)
		{
			gSerialCommunicator->CloseSerialComm();
			delete gSerialCommunicator;
			gSerialCommunicator = NULL;
		}
		
		gSerialCommunicator = TSerialCommunicator::MakeNewSerialConnection(	&gDefaultPortInfoRecord, 
																			&gSelectedPortNameRecord);
		gSerialCommunicator->InitSerialComm();
		
		success = true;
	}
	else
		success = false;
	
	return success;
}

//----------------------------------------------------------------------
//	MacDialogBoxEventHandler()
//
//	Purpose:	Handle events in the configure dialog box 
//					
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------
pascal OSStatus MacDialogBoxEventHandler (	EventHandlerCallRef myHandler, 
											EventRef event, void * userData)
{
    OSStatus 		result = eventNotHandledErr;
    HICommand		command;
    pBoolean		stopModalLoop = false;

	//ControlRef		setTimeAndLocRadioControl; 
	//ControlID 		setTimeAndLocRadioControlID; // removed until further notice

	ControlRef		scopeListControl, portListControl;
    ControlID 		scopeListPopopMenuID, portListPopopMenuID;
	WindowRef		dialogWindowRef = (WindowRef) userData;

	// Find our set time and location checkbox control
    // setTimeAndLocRadioControlID.signature = kPluginApplicationSignature;
    // setTimeAndLocRadioControlID.id = kSetTimeLocRadioID;

	// Find our scope list popup control
	scopeListPopopMenuID.signature = kPluginApplicationSignature;
    scopeListPopopMenuID.id = kScopeTypePopupMenuID;

	// Find our port list popup control
    portListPopopMenuID.signature = kPluginApplicationSignature;
    portListPopopMenuID.id = kCommPortPopupMenuID;
	
	//::GetControlByID(dialogWindowRef, &setTimeAndLocRadioControlID, &setTimeAndLocRadioControl);
	::GetControlByID(dialogWindowRef, &scopeListPopopMenuID, &scopeListControl);
	::GetControlByID(dialogWindowRef, &portListPopopMenuID, &portListControl);

	// Get the HI Command
    ::GetEventParameter (event, kEventParamDirectObject, typeHICommand, NULL, sizeof (HICommand), NULL, &command);
							
    // Look for our 'ok ' and 'not!' commands
    switch (command.commandID)
    {
        case 'ok  ':
				gScopeConfigureBox->SetGlobalsToSelectedScopeAndPort();
				stopModalLoop = true;
                result = noErr;
                break;
        case 'not!':
                stopModalLoop = true;
                result = noErr;
                break;
		default:
				break;
		/*
        case 'Tele':
				gSelectedScopeType = (scopeType) ::GetControlValue(scopeListControl);
				gCanSetScopeTimeAndLocation = TScope::DoesScopeTypeSupportSetTimeAndLocation(gSelectedScopeType);
				if (gCanSetScopeTimeAndLocation)
					::EnableControl(setTimeAndLocRadioControl);
				else
				{
					SInt32 currVal = 0;
					currVal = ::GetControl32BitValue(setTimeAndLocRadioControl);
					::SetControl32BitValue(setTimeAndLocRadioControl, 3);
					::DisableControl(setTimeAndLocRadioControl);
				}
				break;
		*/
    }
	   
    // Stop the modal loop.
    if (stopModalLoop)
        ::QuitAppModalLoopForWindow((WindowRef)userData);

    //Return how we handled the event.
    return result;
}

