/*
 *  UtilityTypedefsHack.h
 *  ScopeConfigUtility
 *
 *  Created by Dave Whipps on Tue Feb 26 2002.
 *  Copyright (c) 2001 SPACE.com Canada Inc. All rights reserved.
 *
 */

// Ensure that this only gets done for the scope test utility
#if TARGET_SCOPE_CONFIG_UTIL

#ifndef utilityTypedefsHack_h
#define utilityTypedefsHack_h

// Need to define some of the structs that are defined in StarryNightPlugins.h since it is 
// not included in the Scope Test Utility build target. 
#ifndef qBuildingPlugin

	typedef Boolean 		pBoolean;
	typedef Rect 			pRect;
	typedef Str255 			pStr255;
	typedef GWorldPtr 		pGWorldPtr;
	typedef Point 			pPoint;
	typedef EventRecord* 	pEventRecordPtr;
	typedef MenuHandle 		pMenuHandle;
	typedef DragReference 	pDragReference;
	typedef ItemReference 	pItemReference;
	typedef PicHandle		pHandleToPicture;
	typedef unsigned long 	sn_WindowRef;
	// PLocation
	typedef struct
	{
		pStr255			fPlaceName;				// eg. "Toronto"
		double			fEarthLatitude;			// in radians
		double			fEarthLongitude;		// in radians
		double			fElevation;				// in metres
		long			fTimeZoneSeconds;		// seconds off of GMT
		unsigned long	fPrivate;				// A private number for Sienna Use only- 
		sn_WindowRef	fInWindow;				// Caution: supported in kVersion7Plugin and later only
	} PLocation;
	
	// PGaze
	typedef struct
	{
		double			fGazeAzimuth;			// "real world" (see sn_Mathematical2RealWorldAzimuth) azimuth of user's gaze (in radians)
		double			fGazeAltitude;			// "real world" (see sn_Mathematical2RealWorldAzimuth) altitude of user's gaze (in radians)
		double			fPixelsPerRadian;		// magnification
		double			fGazeRa;				// the right ascension of the gaze in the epoch of date (not j2000)
		double			fGazeDec;				// the declination of the gaze in the epoch of date
		sn_WindowRef	fInWindow;				// Caution: supported in kVersion7Plugin and later only
	} PGaze;
	
	// PSelection
	typedef struct
	{
		pStr255			fObjectName;			// default name of object
		short			fObjectIconID;			// icon id for icon that represents object (or 0 if none)
		pStr255			fConstellationName;		// name of constellation that object is in
		double			fAz;					// "real world" (see sn_Mathematical2RealWorldAzimuth) azimuth of mouse or object (in radians)
		double			fAlt;					// "real world" (see sn_Mathematical2RealWorldAzimuth) altitude of mouse or object (in radians)
		double			fRightAscension;		// RA of mouse or object (in radians) using epoch of current date
		double			fDeclination;			// declination of mouse or object (in radians) using epoch of current date
		pStr255			fObjectType;			// string describing the object's kind
		double			fMagnitude;				// magnitude if object supports magnitude (otherwise kUndefinedMagnitudeValue)
		double			fAngularSize;			// angular size if object supports ang size (otherwise kUndefinedAngleValue)
		pStr255			fUserFormattedName;		// name of object as user has decided they want it displayed
												// (eg. perhaps "Alpha" instead of Sirius) May be same as fObjectName.
		unsigned long	fObjectKey;				// a key for the object, made up by the object, that is a fairly unique 
												// 32bit reference for the object. .
		long			fObjectLayerID;			// The layer ID of the object selected. The layer ID is a different number for each layer
		unsigned long	fPrivate1;				// A private number for Sienna Use only- Do not change any values passed in here 
		unsigned long	fPrivate2;				// A private number for Sienna Use only- Do not change any values passed in here 
		pBoolean		fObjectIsSelectable;	// true if the object is a selectable entity in Starry Night
		pBoolean		fIsNewPosition;			// only set for op_CursorPosition: true if cursor has moved
	} PSelection;
	
	// PTime
	typedef struct
	{
		double			fJulianDate;		// universal time
		unsigned long	fPrivate;			// A private number for Sienna Use only- Do not change any values passed in here 
	} PTime;
	
	// NOTE that this is a bit of a kludge and should e taken with a grain of salt
	void sn_Julian2CivilTime(double inJulianUT, long* outYear, long* outMonth, long* outDay, long* outHour, long* outMinute, long* outSecond);	

#endif /*qBuildingPlugin*/

#endif /*utilityTypedefsHack_h*/

#endif /*TARGET_SCOPE_CONFIG_UTIL*/
