/*
    Some comments...
*/

#include "MainWindowHandler.h"

#ifndef scope_h
	#include "Scope.h"
#endif /*scope_h*/

OSStatus CreateMainWindow()
{
    IBNibRef 		nibRef;
    EventTypeSpec 	commandSpec = {kEventClassCommand, kEventCommandProcess };
    OSStatus		err;
	
	ControlRef		sendTextControl;
    ControlID 		sendTextControlID;
	
	// Input our send text control ID params
    sendTextControlID.signature = kScopeConfigUtilApplicationSignature;
    sendTextControlID.id = kSendStringEditTextBoxID;
	
    // Create a Nib reference passing the name of the nib file (without the .nib extension)
    // CreateNibReference only searches into the application bundle.
    err = CreateNibReference(CFSTR("ScopeTestUtilMain"), &nibRef);
    require_noerr( err, CantGetNibRef );
    
    // Once the nib reference is created, set the menu bar. "MainMenu" is the name of the menu bar
    // object. This name is set in InterfaceBuilder when the nib is created.
    err = SetMenuBarFromNib(nibRef, CFSTR("MenuBar"));
    require_noerr( err, CantSetMenuBar );
    
    // Then create a window. "MainWindow" is the name of the window object. This name is set in 
    // InterfaceBuilder when the nib is created.
    err = CreateWindowFromNib(nibRef, CFSTR("MainWindow"), &gMainWindow);
    require_noerr( err, CantCreateWindow );

    // We don't need the nib reference anymore.
    DisposeNibReference(nibRef);
	
	// Get the control reference for the send text box
	err = GetControlByID(gMainWindow, &sendTextControlID, &sendTextControl);
    require_noerr( err, CantFindControl);
    
    // Attach our handler to the window.
    err =  InstallWindowEventHandler (	gMainWindow, NewEventHandlerUPP (MainWindowEventHandler), 
										1, &commandSpec, (void *) gMainWindow, NULL);
	
    // The window was created hidden so show it.
    ShowWindow( gMainWindow );

CantFindControl:
CantCreateWindow:
CantSetMenuBar:
CantGetNibRef:
    
    return err;
}

// Window event handler
pascal OSStatus  MainWindowEventHandler(EventHandlerCallRef myHandler, EventRef event, void *userData)
{
    OSStatus 		result = eventNotHandledErr;
    HICommand		command;

    // Get the HI Command
    GetEventParameter (	event, kEventParamDirectObject, typeHICommand, NULL, 
						sizeof (HICommand), NULL, &command);
   
	 // Look for our SHOW and SEND commands
    switch (command.commandID)
    {
        case kConfigureButtonCommand:
				gScopeConfigureBox = TScopeConfigBox::CreateNewConfigBox();
				gScopeConfigureBox->DisplayAndRunConfigBox(); // Show the configure dialog.
                result = noErr;
                break;
				
		case kSendButtonCommand:
				SendInputStringToScope();
				GetReturnStringFromScope();
				result = noErr;
				break;
    }
    
    //Return how we handled the event.
    return result;
}

OSStatus SendInputStringToScope(void)
{
	OSStatus	err;
	ControlRef	sendTextControl;
    ControlID 	sendTextControlID;
	CFStringRef sendStringRef;
	Size		sendStringSize;
	
	LStr255		sendStringAsLString("\p");
	char		sendStringAsCString[256] = "";
	
	// Find our send text control
    sendTextControlID.signature = kScopeConfigUtilApplicationSignature;
    sendTextControlID.id = kSendStringEditTextBoxID;
	
	err = GetControlByID(gMainWindow, &sendTextControlID, &sendTextControl);
    require_noerr( err, CantFindControl);
	
	if(gSerialCommunicator != NULL)
	{
		// Get the string from the input box, clear the box and send it to the scope.
		
		err = GetControlData(	sendTextControl, kControlEntireControl, kControlEditTextCFStringTag,
								sizeof(CFStringRef), &sendStringRef, &sendStringSize);
			
		bool convertedToCString = CFStringGetCString(sendStringRef, sendStringAsCString, sizeof(sendStringAsCString), kCFStringEncodingMacRoman);
		
		sendStringAsLString.Assign(sendStringAsCString);
		
		if(gSerialCommunicator && convertedToCString && gSerialCommunicator->GetIsSerialConnectionInitialized())
			err = gSerialCommunicator->SendSerialInfo(sendStringAsLString);
		
		CFRelease(sendStringRef);
	}
	
	CantFindControl:
	return err;
}

OSStatus GetReturnStringFromScope(void)
{
	OSStatus	err;
	ControlRef	returnTextControl;
    ControlID 	returnTextControlID;
	CFStringRef returnStringRef = NULL;
	
	LStr255		returnStringAsLString = "";
	
	// Find our return text control
    returnTextControlID.signature = kScopeConfigUtilApplicationSignature;
    returnTextControlID.id = kReturnStringEditTextBoxID;

	err = GetControlByID(gMainWindow, &returnTextControlID, &returnTextControl);
	require_noerr( err, CantFindControl);
	
	if(gSerialCommunicator != NULL)
	{
		// Get the string from the serial port and put into the return string edit text box.
		
		if(gSerialCommunicator && gSerialCommunicator->GetIsSerialConnectionInitialized())
		{
			err = gSerialCommunicator->GetSerialInfo(	returnStringAsLString, 
														kMaxCharsToReadFromScope,
														kReturnStringTimeoutInMilliseconds);
		}
		
		returnStringRef = CFStringCreateWithPascalString(NULL, returnStringAsLString, kCFStringEncodingMacRoman);
		
		err = SetControlData(	returnTextControl, kControlEntireControl, kControlEditTextCFStringTag,
								sizeof(CFStringRef), &returnStringRef);
		
		//Redraw the control immediately to update the text
		Draw1Control(returnTextControl);
		
		CFRelease(returnStringRef);
	}
	
	CantFindControl:
	return err;
}

// The dialog handler will call this function when the user clicks a button.
void HandleResponseFromConfigDialog(Boolean response)
{
    ControlRef 		configureStatusTextControl;
	ControlRef		configureButtonControl;
	ControlRef		sendButtonControl;
    ControlID 		configureStatusTextControlID, configureButtonControlID, sendButtonControlID;
    OSStatus		err;
    CFStringRef		configureStatusText, configureButtonTitleText;
	
    // Find our configure status text control
    configureStatusTextControlID.signature = kScopeConfigUtilApplicationSignature;
    configureStatusTextControlID.id = kConfigureStatusTextID;
    
	// Find our configure status text control
    configureButtonControlID.signature = kScopeConfigUtilApplicationSignature;
    configureButtonControlID.id = kConfigureButtonID;
	
	// Find our configure status text control
    sendButtonControlID.signature = kScopeConfigUtilApplicationSignature;
    sendButtonControlID.id = kSendButtonID;
	
	// Find our status text box control
    statusGroupBoxControlID.signature = kScopeConfigUtilApplicationSignature;
    statusGroupBoxControlID.id = kStatusGroupBoxID;
	
    err = GetControlByID(gMainWindow, &configureStatusTextControlID, &configureStatusTextControl);
    require_noerr( err, CantFindControl);

	err = GetControlByID(gMainWindow, &configureButtonControlID, &configureButtonControl);
	require_noerr( err, CantFindControl);
	
	err = GetControlByID(gMainWindow, &sendButtonControlID, &sendButtonControl);
	require_noerr( err, CantFindControl);
	  
	// Generate a response
    if (response == TRUE)
	{
        configureStatusText = CFSTR(kPortConfiguredString);
		configureButtonTitleText = CFSTR(kReconfigureButtonString);
		
		// Change the title of the 'Configure' button to 'Reconfigure'
		SetControlTitle(configureButtonControl, 
						CFStringGetPascalStringPtr(configureButtonTitleText, kCFStringEncodingMacRoman));
						
		// Enable the 'Send' button and make it the default button
		if(gIsPortConfigured)
		{
			EnableControl(	sendButtonControl);
		}
	}
	else
	{
		if(gIsPortConfigured)
		{
	        configureStatusText = CFSTR(kPortConfiguredString);
			EnableControl(sendButtonControl);
		}
		else
		{
			configureStatusText = CFSTR(kMustConfigureString);
			DisableControl(sendButtonControl);
		}
    }
	
    // Change the configure status string
    SetControlData(	configureStatusTextControl, 
					kControlEntireControl,
					kControlStaticTextCFStringTag,
					sizeof(CFStringRef),
					&configureStatusText);
	
CantFindControl:
    return;
}

