//----------------------------------------------------------------------
//	Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		SerialCommunicator.h
//
//	Contains:	Wrapper classes for the two seperate  types of serial communicators
//								
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		GetSerialInfo can now stop waiting for data when stopFlag
//												is reached
//	01/09/15	DWW			4.0.0		New support for communicating with serial
//
//----------------------------------------------------------------------

#include "GSXSerialCommunicator.h"


class GSXSerialCommunicator;

//----------------------------------------------------------------------
//	Class:		TSerialCommunicator
//
//	Purpose:	Abstract Class allowing for communications through serial ports
//				on either Mac or Win
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/02	MK			5.0.2		Added stopFlag option to GetSerialInfo
//	00/05/29	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

class TSerialCommunicator
{
	public:
	
	// A platform independant method for finding serial ports for use.
		static UInt8	FindReadySerialPorts(GSXSerialPortNameRecordHandle *outPortInfo);
		
	// A platform independant method for creating the serial connection.	
		static TSerialCommunicator*	MakeNewSerialConnection(const GSSerialPortInfoRecord  *inPortInfo,																										const GSXSerialPortNameRecord *inPortName);
															
	public:

						TSerialCommunicator(void);
		virtual 		~TSerialCommunicator(void);

	public:
		// Open - close serial ports						
		virtual SInt32	InitSerialComm(void);
		virtual SInt32	CloseSerialComm(void);

		// options							
		virtual SInt32	SetPortOptions(const GSSerialPortInfoRecord *inPortInfo, const GSXSerialPortNameRecord *inPortName);
				SInt32	GetPortOptions(GSSerialPortInfoRecord *outPortInfo, GSXSerialPortNameRecord *outPortName);

		// Send and Recieve data
		virtual SInt32	SendSerialInfo(const void *inBuffer, const UInt32 inBufferSize);
		virtual SInt32	SendSerialInfo(const LString& inBuffer);

		virtual SInt32	GetSerialInfo(void *outBuffer, UInt32 *ioCount, double inWaitMilliseconds, const char stopFlag);
		virtual SInt32	GetSerialInfo(LStr255& outBuffer, UInt32 maxNumberOfCharsToRead, double inWaitMilliseconds, const char stopFlag);
		
		virtual SInt32	PurgeFlushSerialBuffer(bufferType whichBuffer);
	
		// public accessors
		bool			GetIsSerialConnectionInitialized(void) const;
		void			SetIsSerialConnectionInitialized(bool hasBeenInitialized);
		
	protected:
		bool			fIsSerialConnectionInitialized;
		GSXSerialCommunicator	*fGSXSerialConnection;
};

#if TARGET_OS_MAC
//----------------------------------------------------------------------
//	Class:		TMacOSXSerialCommunicator
//
//	Purpose:	Class allowing messages to be passed to and from serial ports in Mac OSX.
//			
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	00/05/29	DWW			4.0.0		New for Pro 4.0
//
//----------------------------------------------------------------------

class TMacOSXSerialCommunicator : public TSerialCommunicator
{
	public:
		// Constructor.
					TMacOSXSerialCommunicator(	const GSSerialPortInfoRecord *inPortInfo,
												const GSXSerialPortNameRecord *inPortName,
												const bool inAsynchronous);
		// Destructor						
		virtual		~TMacOSXSerialCommunicator(void);					
};

#endif /*TARGET_OS_MAC*/
