//
//	GSSerialMacOS.h
//	 1998, 1999, 2000, 2001 Kyle Hammond
//	kyle@codeblazer.com
//	Use at your own risk.  All rights reserved.  Do not distribute without permission.
//

//----------------------------------------------------------------------
//	Modified by Dave Whipps of Imaginova Canada Inc.
//	Modifications Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		GSXSerialMacOS.h
//
//	Contains:	Class for dealing with Mac OSX serial ports
//							
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	05/03/12	MK			5.02		Reading can now stop when a certain flag character is encountered
//										This could speed up response drastically.
//	01/12/12	DWW			4.0.0		New internal telescope support for Pro 4
//
//----------------------------------------------------------------------

#include "GSXSerialCommunicator.h"


#define kGSSerialMacBufferSize			256

// Can only use this class from MacOS X (Carbon)
class GSSerialMacOSX : public GSXSerialCommunicator 
{
	public:
		// Constructor.
		explicit GSSerialMacOSX(const GSSerialPortInfoRecord *inPortInfo, const GSXSerialPortNameRecord *inPortName,
								const bool inAsynchronous) : GSXSerialCommunicator( inPortInfo, inPortName, inAsynchronous ), mFileDescriptor( -1 ) { }

		virtual SInt32	InitSerialComm( void );
		virtual SInt32	CloseSerialComm( void );

		virtual SInt32	SetPortOptions( const GSSerialPortInfoRecord *inPortInfo,
												const GSXSerialPortNameRecord *inPortName );

		virtual SInt32	SendSerialInfo( const void *inBuffer, const unsigned long inBufferSize );
		virtual SInt32	GetSerialInfo( void *outBuffer, unsigned long *ioCount, const long inWaitTicks, const char stopFlag );

		virtual SInt32	PurgeFlushSerialBuffer(bufferType whichBuffer);
                
		// Class functions.
		static SInt8	FindReadySerialPorts( GSXSerialPortNameRecordHandle *outPortInfo);

	// Implementation section.
	private:
		// Hold the original termios attributes so we can reset them
		struct termios		mOriginalTTYAttrs;
		SInt16				mFileDescriptor;

		// Can't use default constructor or copy constructor.
		GSSerialMacOSX( );
		GSSerialMacOSX( const GSSerialMacOSX & );
};
