//----------------------------------------------------------------------
//	Copyright  1990-2005 by Imaginova Canada, Inc., All rights reserved.
//
//	File:		ControllerPluginUtils.cpp
//
//	Contains:	Helper Utils for scope controller plugin.
//					
//	Authors:	Dave Whipps
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	01/09/18	DWW			4.0.0		New for Pro 4.0
//----------------------------------------------------------------------

#include "ControllerPluginUtils.h"

///////////////////////////////////////////////////////////////////////////
// Telescope status messages and informative text						 //
///////////////////////////////////////////////////////////////////////////
const LStr255 kMessage_ConnectionClosed("\pConnection Closed");
const LStr255 kInformative_ConnectionClosed("\pThe connection to the telescope is closed. To open the connection, click \"Connect\".");

const LStr255 kMessage_GenericError("\pAn Error Has Occured");
const LStr255 kInformative_GenericError("There has been an undetermined error communicating with the telescope. Ensure that all cables are peoperly connected and that you have properly configured your scope and serial connection.");

const LStr255 kMessage_SNLocationChanged("\pLocation Changed");
const LStr255 kInformative_SNLocationChanged("\pYour Starry Night location has changed. This may affect the collaboration between the telescope plugin and Starry Night. In order to ensure corrrect operation, close and reopen the telescope connection.");

const LStr255 kMessage_CommunicationError("\pCommunication Error");
const LStr255 kInformative_CommunicationError("\pAn error has occured while trying to communicate with the scope. Ensure that all cables are properly connected and that you have correctly configured for your scope and serial connection.");

const LStr255 kMessage_BelowHorizon("\pCoordinates Below Horizon");
const LStr255 kInformative_BelowHorizon("\pThe coordinates you have chosen to slew or synch to are below the telescopes horizon. Ensure that the location and time set in the telescopes memory are the same as those in Starry Night.");

const LStr255 kMessage_LX200Specific("\pLX200 Error");
const LStr255 kInformative_LX200Specific("\pThere has been an error specific to the LX200 type scopes. Ensure that all of the telescope location an time settings are correct and that the scope is correctly plugged in.");


// We'll want to replace the <ERROR> constant strings with the actual 
// values, so the aggregate strings and PPluginStatus can't be constants.

const	LStr255 kErrorStringToReplace("\p<ERROR>");

const	LStr255 kMessage_SpecificError("\pAn Error Has Occured");
const	LStr255 vInformative_SpecificError("\p<ERROR>");		

const	LStr255 kMessage_Idle("\pTelescope Idle");
const	LStr255 kInformative_Idle("\pThe telescope is idle.");

const	LStr255 kMessage_Tracking("\pTracking");
const	LStr255 kInformative_Tracking("\pThe telescope is tracking across the sky at it's internally set rate.");
		
const	LStr255 kMessage_Following("\pFollowing Telescope");
const	LStr255 kInformative_Following("\pStarry Night is currently following the movements of the telescope. To return to normal control, uncheck the \"Follow Scope\" checkbox.");

const   LStr255 kMessage_Reporting("\pReporting telescope position");
const   LStr255 kInformative_Reporting("\pStarry Night is currently reporting the position of the telescope.");
		
const	LStr255 kMessage_ScopeAlreadySlewing("\pScope Slewing");
const	LStr255 kInformative_ScopeAlreadySlewing("\pThe telescope is currently performing a slew. Your slew request has been ignored. Wait until the scope is finished moving and make the request again.");

const	LStr255 kMessage_Synching("Synching");
const	LStr255 kInformative_Synching("The telescope has been synchronized to the centre of the SN gaze");

//----------------------------------------------------------------------
//	ClockSec
//
//	Purpose:	returns time since computer started or some other arbitrary start, in seconds 
//					
//	Inputs:		none		
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/01	Tom			3.0			Cross platform telescope controller
//
//----------------------------------------------------------------------
double ClockSec(void)
{
	double rawTime = clock();
	double timeSec = rawTime/CLOCKS_PER_SEC;
	return timeSec;
}

//----------------------------------------------------------------------
//	PStringAssign
//
//	Purpose:	assigns a c string to a pascal string.
//					
//	Inputs:		source			source string
//				destination		destination string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------
 unsigned char* PStringAssign(unsigned char* destination, const char* source)
{
	destination[0] = strlen(source);
	for (short charPos = destination[0] + 1; charPos--; destination[charPos+1] = source[charPos]);
	return (destination);
}

//----------------------------------------------------------------------
//	PStringAssign
//
//	Purpose:	assigns a c string to a pascal string.
//					
//	Inputs:		source			source string
//				destination		destination string
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	98/07/29	ETL			1.0			New
//
//----------------------------------------------------------------------
unsigned char* PStringAssign(unsigned char* destination, LStr255 *source)
{
	destination[0] = source->Length();
	for (short charPos = destination[0] + 1; charPos--; destination[charPos+1] = (*source)[charPos+1]);
	return (destination);
}

//----------------------------------------------------------------------
//	HandleError
//
//	Purpose:	Called when an actual error occurs
//					
//	Inputs:		inSNErr	description of error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/18	Tom			3.0			New Callback API
//
//----------------------------------------------------------------------
void HandleError(long inSNErr)
{
	switch (inSNErr)
	{
		case snNoErr:
			break;
		case pluginErr_CommunicationError:
			::SetStatusText(&kMessage_CommunicationError, &kInformative_CommunicationError);
			break;
		case pluginErr_Failed:
			::SetStatusText(&kMessage_GenericError, &kInformative_GenericError);
			break;
		case pluginErr_ObjectBelowHorizon:
			::SetStatusText(&kMessage_BelowHorizon, &kInformative_BelowHorizon);
			break;
		case pluginErr_LX200SpecificError:
			::SetStatusText(&kMessage_LX200Specific, &kInformative_LX200Specific);
			break;
		case pluginErr_SlewAlreadyInProgress:
			::SetStatusText(&kMessage_ScopeAlreadySlewing, &kInformative_ScopeAlreadySlewing);
			break;		
		default:
			::SetStatusText(&kMessage_GenericError, &kInformative_GenericError);
			break;
	}
}


//----------------------------------------------------------------------
//	CallBackErrorHandler
//
//	Purpose:	This call is an outline of how you might handle an erorr 
// 				coming back from one of the Sienna API calls
//				In this version, I just have a place to put a break point
//				in with the debugger, but you could do anything.
//					
//	Inputs:		inSNErr	description of possible error
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	99/02/18	Tom			3.0			New Callback API
//
//----------------------------------------------------------------------
void CallBackErrorHandler(long inSNErr)
{
	if (inSNErr != snNoErr)
		::HandleError(inSNErr);
}

//----------------------------------------------------------------------
//	SetStatusText
//
//	Purpose:	Sets the status strings in the Starry Night "Telescope" Panel
//	
//	Inputs:		inStatus		A pointer to the status struct to be sent to Starry Night
//				inRA			The current right ascension of the scope in decimal radians
//				inDec			The current declination of the scope in decimal radians
//				inErrorStringPtr A pointer to the error string to be added to the status
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	02/03/04	DWW			4.0			New for Pro X
//----------------------------------------------------------------------
void SetStatusText(const LStr255* inMessageText, const LStr255* inInformativeText, double inRA, double inDec)
{
	LStr255 messageTextCopy(*inMessageText);
	LStr255 informativeTextCopy(*inInformativeText);
	
	::sn_SetPluginStatusText(messageTextCopy, informativeTextCopy, inRA, inDec);
}