// =======================================================================================
//	PluginLString.cp			*MODIFIED* PowerPlant 2.2		1995-2001 Metrowerks Inc.
// =======================================================================================
//
//	Classes for handling Pascal Strings
//
//	*******************************************************
//	NOTE: 	THIS FILE HAS BEEN MODIFIED FROM ITS ORIGINAL
//			FORM TO FIT WITH THE REQUIREMENTS OF THE STARRY
//			NIGHT TELESCOPE CONTROLLER PLUGIN.
//	******************************************************* 
//
//	LString		- Abstract base class
//	LStr255		- Wrapper class for Str255
//	TString<T>	- Template class for any string type (array of unsigned char)

//#include <fp.h>
#include "PluginLString.h"


// ===========================================================================
//	 LString
// ===========================================================================
//	Abstract base class for a Pascal-style string

// ---------------------------------------------------------------------------
//		 operator SInt32
// ---------------------------------------------------------------------------
//	Conversion to a long integer

LString::operator SInt32() const
{
	if (Length() == 0) // no string means 0, That is what I want, and the StringToNum docs are unclear on the issue
		return 0;

	SInt32	theNumber;
	
	// Old code: used StringtoNum, StringToNum sucks.
	unsigned char cString[256];
	CopyPStr(mStringPtr, cString);
	PToCStr(cString);	
	
	sscanf((const char*) &cString[0], "%d", &theNumber);
	
	//::StringToNum(mStringPtr, &number);
	return theNumber;
}


#pragma mark --- Assignment ---

// ---------------------------------------------------------------------------
//		 Assign ( LString&, UInt8, UInt8 )
// ---------------------------------------------------------------------------
//	Assigment from a substring of another String object
//
//	Use inStartPos and inCount to specify the substring of inString to
//	copy into this String. The default values of these parameters
//	will copy all of inString.
//
//	If inStartPos is beyond the end of inString or inCount is zero,
//	this String is made empty. If inCount extends beyong the end of
//	inString, only bytes from inStartPos to the end of inString
//	are copied.

LString&
LString::Assign(
	const LString&	inString,
	UInt8			inStartPos,
	UInt8			inCount)
{
	if ((inStartPos > inString.Length()) || (inCount == 0)) {
										// Substring to copy is empty
		mStringPtr[0] = 0;				//   so make this String empty

	} else {
										// If count is too big, limit it
										//   to the end of the String
		if ((inStartPos + inCount - 1) > inString.Length()) {
			inCount = (UInt8) (inString.Length() - inStartPos + 1);
		}

		LoadFromPtr(&inString[inStartPos], inCount);
	}

	return *this;
}


// ---------------------------------------------------------------------------
//		 Assign ( ConstStringPtr )
// ---------------------------------------------------------------------------
//	Assigment from a Pascal string

LString&
LString::Assign(
	ConstStringPtr	inStringPtr)
{
	if (inStringPtr == NULL) {
		mStringPtr[0] = 0;

	} else {
		Assign(inStringPtr + 1, inStringPtr[0]);
	}

	return *this;
}


// ---------------------------------------------------------------------------
//		 Assign ( unsigned char )
// ---------------------------------------------------------------------------
//	Assigment from a character

LString&
LString::Assign(
	unsigned char	inChar)
{
	mStringPtr[0] = 1;					// 1-character string
	mStringPtr[1] = inChar;
	return *this;
}


// ---------------------------------------------------------------------------
//		 Assign ( const void*, UInt8 )
// ---------------------------------------------------------------------------
//	Set String contents from a pointer to characters and a length

LString&
LString::Assign(
	const void*		inPtr,
	UInt8			inLength)
{
	LoadFromPtr(inPtr, inLength);
	return *this;
}

// ---------------------------------------------------------------------------
//		 Assign ( SInt32 )
// ---------------------------------------------------------------------------
//	Assignment from a long integer

LString&
LString::Assign(SInt32 inNumber)
{
	char cString[256];
	sprintf(cString, "%d", inNumber);
	
	Assign(cString);
	return *this;
}


/*
#if TARGET_OS_MAC					// Only works for Mac OS

// ---------------------------------------------------------------------------
//	The conversion routine num2dec() in <fp.h> takes a double_t parameter.
//	The basic type of double_t is "long double" on 68K, but "double" on
//	PPC (and other platforms). The binary MathLib library which implements
//	num2dec() depends on these definitions.
//
//	Unfortunately, both <fp.h> and <cmath> define double_t. If the definitions
//	differ, num2dec() will probably fail to work properly. To enforce the
//	correct type and avoid conflicting definitions, we define our own type.

#if TARGET_CPU_68K
	typedef long double		fp_double_t;
#else
	typedef double			fp_double_t;
#endif

// ---------------------------------------------------------------------------
//	 Num2Dec2Str
// ---------------------------------------------------------------------------

static void
Num2Dec2Str(
	LString*	inString,
	fp_double_t	inNumber,
	char		inStyle,
	SInt16		inDigits)
{
        // Toolbox num2dec() converts a double_t number to a
        // decform, a struct which describes ae number

	decform	theDecform;
	theDecform.style  = (char) inStyle;
	theDecform.digits = inDigits;
	decimal	decNumber;

	::num2dec(&theDecform, inNumber, &decNumber);

		// The Toolbox dec2str() routine takes a "decimal" number
		// and converts it into a C string that can be at most
		// DECSTROUTLEN characters (we add 1 for the null terminator).

	char	numStr[DECSTROUTLEN + 1];
	::dec2str(&theDecform, &decNumber, numStr);

	inString->Assign(numStr, LString::CStringLength(numStr));
}

#endif // TARGET_OS_MAC
*/

#pragma mark --- Append ---

// ---------------------------------------------------------------------------
//		 Append ( unsigned char )
// ---------------------------------------------------------------------------
//	Append a character to this String

LString&
LString::Append(
	unsigned char	inChar)
{
	if (mStringPtr[0] < (mMaxBytes - 1)) {
		mStringPtr[++mStringPtr[0]] = inChar;
	}

	return *this;
}


// ---------------------------------------------------------------------------
//		 Append ( SInt32 )
// ---------------------------------------------------------------------------
//	Append a number to this String

LString&
LString::Append(SInt32 inNumber)
{
	char cString[256];
	sprintf(cString, "%d", inNumber);

	return Append(cString);
}


// ---------------------------------------------------------------------------
//		 Append ( const void*, UInt8 )
// ---------------------------------------------------------------------------
//	Append characters to the end of this String

LString&
LString::Append(
	const void*		inPtr,
	UInt8			inLength)
{
	SInt16	charsToCopy = inLength;			// Limit to max size of String
	if ((mStringPtr[0] + charsToCopy + 1) > mMaxBytes) {
		charsToCopy = (SInt16) (mMaxBytes - mStringPtr[0] - 1);
	}

	memmove( (mStringPtr + mStringPtr[0] + 1), inPtr, charsToCopy);
	//::BlockMoveData(inPtr, mStringPtr + mStringPtr[0] + 1, charsToCopy );

	mStringPtr[0] += (UInt8) charsToCopy;	// Adjust length byte

	return *this;
}

#pragma mark --- Search for Substring ---

// ---------------------------------------------------------------------------
//		 Find ( const void*, UInt8, UInt8 )
// ---------------------------------------------------------------------------
//	Return the index of where a substring starts within this String
//
//	Returns 0 if the substring is not in the String
//	inStartPos is the index within the String at which to begin searching.
//		The default value is 1. Value 0 is not valid.
//
//	To find the next occurrence, set inStartPos to one plus the index
//	returned by the last Find.

UInt8
LString::Find(
	const void*		inPtr,				// Ptr to first char of string to find
	UInt8			inLength,			// Length of string to find
	UInt8			inStartPos) const
{
	UInt8	startIndex = 0;
										// Last possible index depends on
										//   relative lengths of the strings
	SInt16	lastChance = (SInt16) (Length() - inLength + 1);

										// Search forward from starting pos
	for (SInt16 i = inStartPos; i <= lastChance; i++) {
		if ((*mCompareFunc)(mStringPtr + i, inPtr, inLength, inLength) == 0) {
			startIndex = (UInt8) i;
			break;
		}
	}

	return startIndex;
}


// ---------------------------------------------------------------------------
//		 ReverseFind ( const void*, UInt8, UInt8 )
// ---------------------------------------------------------------------------
//	Return the index of where a substring starts within this String, searching
//	from the end of the String
//
//	Returns 0 if the substring is not in the String
//	inStartPos is the index within the String at which to begin searching.
//		The default value is 255. Value 0 is not valid.
//
//	To find the previous occurrence, set inStartPos to the index returned
//	by the last ReverseFind minus 1.

UInt8
LString::ReverseFind(
	const void*		inPtr,
	UInt8			inLength,
	UInt8			inStartPos) const
{
	UInt8	startIndex = 0;
										// Highest possible index depends on
										//   relative lengths of the strings
	SInt16	lastChance = (SInt16) (Length() - inLength + 1);
	if (lastChance > 0) {				// Substring can't be longer
		if (inStartPos > lastChance) {	// Don't bother looking after lastChance
			inStartPos = (UInt8) lastChance;
		}
										// Search backward from starting pos
		for (SInt16 i = inStartPos; i >= 1; i--) {
			if ((*mCompareFunc)(mStringPtr + i, inPtr, inLength, inLength) == 0) {
				startIndex = (UInt8) i;
				break;
			}
		}
	}

	return startIndex;
}


// ---------------------------------------------------------------------------
//		 BeginsWith ( const void*, UInt8 )
// ---------------------------------------------------------------------------
//	Return whether this String begins with a specified String

Boolean
LString::BeginsWith(
	const void*		inPtr,
	UInt8			inLength) const
{
										// Compare first "inLength" bytes
										//   of String with input
	return (inLength <= Length()) &&
		   ((*mCompareFunc)(mStringPtr + 1, inPtr, inLength, inLength) == 0);
}


// ---------------------------------------------------------------------------
//		 EndsWith ( const void*, UInt8 )
// ---------------------------------------------------------------------------
//	Return whether this String ends with a specified String

Boolean
LString::EndsWith(
	const void*		inPtr,
	UInt8			inLength) const
{
										// Compare last "inLength" bytes
										//   of String with input
	return (inLength <= Length()) &&
		   ((*mCompareFunc)(mStringPtr + Length() - inLength + 1, inPtr,
		   					inLength, inLength) == 0);
}


// ---------------------------------------------------------------------------
//		 FindWithin ( const void*, UInt32, UInt8 )
// ---------------------------------------------------------------------------
//	Return the index of where this String starts within another String
//
//	Returns 0 if the substring is not in the String
//	inStartPos is the index within inPtr at which to begin searching.
//		The default value is 1. Value 0 is not valid.
//
//	To find the next occurrence, set inStartPos to one plus the index
//	returned by the last FindWithin.

UInt32
LString::FindWithin(
	const void*		inPtr,				// Ptr to first char of string to search
	UInt32			inLength,			// Length of string to search
	UInt32			inStartPos) const	// Index at which to start searching
{
	UInt32	startIndex = 0;

										// inStartPos of zero is not valid and
										//   substring can't be longer
	if ( (inStartPos > 0) && (inLength >= Length()) ) {

										// Highest possible offset depends on
										//   relative lengths of the strings
		UInt32	lastOffset = inLength - Length();

										// Search forward after starting pos
		for (UInt32 offset = inStartPos - 1; offset <= lastOffset; offset++) {
			if ((*mCompareFunc)(mStringPtr + 1, (UInt8*) inPtr + offset, Length(), Length()) == 0) {
				startIndex = offset + 1;
				break;
			}
		}
	}

	return startIndex;
}


// ---------------------------------------------------------------------------
//		 ReverseFindWithin ( const void*, UInt32, UInt8 )
// ---------------------------------------------------------------------------
//	Return the index of where this String starts within another String,
//	searching from the end of the String
//
//	Returns 0 if this String is not within the input string
//	inStartPos is the index within inPtr at which to begin searching.
//		Value 0 is not valid.
//
//	To find the previous occurrence, set inStartPos to the index returned
//	by the last ReverseFindWithin minus 1.

UInt32
LString::ReverseFindWithin(
	const void*		inPtr,
	UInt32			inLength,
	UInt32			inStartPos) const
{
	UInt32	startIndex = 0;

										// inStartPos of zero is not valid and
										//   substring can't be longer
	if ( (inStartPos > 0) && (inLength >= Length()) ) {
										// Highest possible offset depends on
										//   relative lengths of the strings
		UInt32	lastOffset = inLength - Length();

										// We start from the lesser of lastOffset
		if (inStartPos <= lastOffset) {	//   and inStartPos - 1
			lastOffset = inStartPos - 1;
		}

										// Search backward from last offset
		UInt32	offset = lastOffset + 1;
		do {
			if ((*mCompareFunc)(mStringPtr + 1, (UInt8*) inPtr + (--offset), Length(), Length()) == 0) {
				startIndex = offset + 1;
				break;
			}
		} while (offset > 0);
	}

	return startIndex;
}

#pragma mark --- Copy Substring ---

// ---------------------------------------------------------------------------
//		 operator () ( UInt8, UInt8 )
// ---------------------------------------------------------------------------
//	Return a LStr255 String object containing the specified substring
//	of this String

LStr255
LString::operator()(
	UInt8	inStartPos,
	UInt8	inCount) const
{
	LStr255	subString;

	if (inStartPos <= Length()) {
		if (inStartPos == 0) {			// Interpret 0 or 1 as the first
			inStartPos = 1;				//    character
		}

										// If count is too big, limit it
										//   to the end of the String
		if ((inStartPos + inCount - 1) > Length()) {
			inCount = (UInt8) (Length() - inStartPos + 1);
		}

		subString.Assign(mStringPtr + inStartPos, inCount);
	}

	return subString;
}

#pragma mark --- Changing Substrings ---

// ---------------------------------------------------------------------------
//		 Insert ( const void*, UInt8, UInt8 )
// ---------------------------------------------------------------------------
//	Insert a string into this String starting at the specified index

LString&
LString::Insert(
	const void*		inPtr,
	UInt8			inLength,
	UInt8			inAtIndex)
{
	if (inAtIndex == 0) {				// Interpret 0 or 1 as the start
		inAtIndex = 1;					//   of the String
	} else if (inAtIndex > Length() + 1) {
										// Interpret any index past end
										//   to mean to append the string
		inAtIndex = (UInt8) (Length() + 1);
	}

										// Don't insert more than bytes
										//    than this String can hold
	if ((inLength + Length() + 1) > mMaxBytes) {
		inLength = (UInt8) (mMaxBytes - Length() - 1);
	}

	if (inAtIndex <= Length()) {		// Shift up bytes after insertion
		memmove( (mStringPtr + inAtIndex + inLength), (mStringPtr + inAtIndex), (Length() - inAtIndex + 1));
		//::BlockMoveData(mStringPtr + inAtIndex,
		//				mStringPtr + inAtIndex + inLength,
		//				Length() - inAtIndex + 1);
	}

										// Copy input string
	memmove((mStringPtr + inAtIndex), inPtr, inLength);
	//::BlockMoveData(inPtr, mStringPtr + inAtIndex, inLength);

	mStringPtr[0] += inLength;			// Adjust length byte

	return *this;
}


// ---------------------------------------------------------------------------
//		 Remove ( UInt8, UInt8 )
// ---------------------------------------------------------------------------
//	Remove a given number of bytes from this String starting at the
//	specified index

LString&
LString::Remove(
	UInt8	inStartPos,
	UInt8	inCount)
{
	if (inStartPos <= Length()) {
		if (inStartPos == 0) {			// Interpret 0 or 1 as the first
			inStartPos = 1;				//    character
		}

										// If count is too big, limit it
										//   to the end of the String
		if ((inStartPos + inCount - 1) > Length()) {
			inCount = (UInt8) (Length() - inStartPos + 1);
		}

		if ((inStartPos + inCount) <= Length()) {
										// Shift down bytes after removed one
			memmove((mStringPtr + inStartPos), (mStringPtr + inStartPos + inCount), (Length() - inStartPos - inCount + 1));
			//::BlockMoveData(mStringPtr + inStartPos + inCount,
			//				mStringPtr + inStartPos,
			//				Length() - inStartPos - inCount + 1);
		}

		mStringPtr[0] -= inCount;		// Adjust length byte
	}

	return *this;
}


// ---------------------------------------------------------------------------
//		 Replace ( UInt8, UInt8, const void*, UInt8 )
// ---------------------------------------------------------------------------
//	Replace a given number of bytes in this String starting at the
//	specified index with the another string, specified a pointer and length

LString&
LString::Replace(
	UInt8			inStartPos,			// First byte to replace
	UInt8			inCount,			// Number of bytes to replace
	const void*		inPtr,				// Ptr to replacement bytes
	UInt8			inLength)			// Number of replacement bytes
{
	Remove(inStartPos, inCount);		// Replace same as Remove & Insert
	Insert(inPtr, inLength, inStartPos);

	return *this;
}

#pragma mark --- Constructor/Destructor ---

// ---------------------------------------------------------------------------
//		 LString
// ---------------------------------------------------------------------------
//	Constructor from a max length and a string pointer
//
//	Subclasses must call this "protected" constructor to specify the
//	maximum string size (including the length byte) and a pointer to
//	the storage for the string

LString::LString(
	SInt16		inMaxBytes,
	StringPtr	inStringPtr)
{
	mMaxBytes = (UInt16) inMaxBytes;
	mStringPtr = inStringPtr;
	mCompareFunc = CompareBytes;  // default changed to compare bytes 
}


// ---------------------------------------------------------------------------
//		 ~LString
// ---------------------------------------------------------------------------
//	Destructor is pure virtual, but we must implement it anyway

LString::~LString()
{
}

#pragma mark --- Comparison Functions ---


// ---------------------------------------------------------------------------
//		 CompareBytes [static]
// ---------------------------------------------------------------------------
//	Simple byte-by-byte value comparison of two strings

SInt16
LString::CompareBytes(
	const void*		inLeft,
	const void*		inRight,
	UInt8			inLeftLength,
	UInt8			inRightLength)
{
	UInt8	n = inLeftLength;				// Determine shortest length
	if (inRightLength < inLeftLength) {
		n = inRightLength;
	}

	const UInt8	*leftP = (UInt8*) inLeft;
	const UInt8 *rightP = (UInt8*) inRight;

	while (n > 0) { 						// Compare up to shortest length
		if (*leftP != *rightP) {
			if (*leftP > *rightP) {
				return 1;					// Left is greater
			}
			return -1;						// Left is smaller
		}

		leftP++;
		rightP++;
		n--;
	}

		// All bytes up to shortest length match. Therefore, the
		// longer of the two string is the greater one. If they
		// have the same length, then they are equal.

	if (inLeftLength > inRightLength) {
		return 1;

	} else if (inLeftLength < inRightLength) {
		return -1;
	}

	return 0;								// String are equal
}

#pragma mark --- Public Utilities ---

// ---------------------------------------------------------------------------
//		 CStringLength
// ---------------------------------------------------------------------------
//	Return the length of a C string, but with an upper limit of 255

UInt8
LString::CStringLength(
	const char*		inCString)
{
		// Find length of C string by searching for the terminating
		// null character. However, don't bother to look past 255
		// characters, since that's all that a Pascal string can hold.

	UInt8	strLength = 0;

	if (inCString != NULL) {
		while ( (strLength < 255)  &&  (inCString[strLength] != '\0') ) {
			strLength += 1;
		}
	}

	return strLength;
}


// ---------------------------------------------------------------------------
//	 CopyPStr
// ---------------------------------------------------------------------------
//	Copy a Pascal string
//
//	Returns a pointer to the copied string
//
//	inSourceString may be nil, in which case outDestString is set to
//	a zero length string
//
//	outDestString must not be nil
//
//	inDestSize specifies the maximum size of the destination (INCLUDING
//		the length byte). It must be >= 1. The default is sizeof Str255.
//
//	Call this function as follows:
//
//		CopyPStr(source, dest, sizeof(dest));

StringPtr
LString::CopyPStr(
	ConstStringPtr	inSourceString,
	StringPtr		outDestString,
	SInt16			inDestSize)
{
	SInt16	dataLen = 1;

	if (inSourceString != NULL) {
		dataLen = (SInt16) (inSourceString[0] + 1);
		if (dataLen > inDestSize) {
			dataLen = inDestSize;
		}
		
		memmove(outDestString, inSourceString, dataLen);
		//::BlockMoveData(inSourceString, outDestString, dataLen);
	}

	outDestString[0] = (UInt8) (dataLen - 1);		// Store length byte

	return outDestString;
}


// ---------------------------------------------------------------------------
//	 CToPStr
// ---------------------------------------------------------------------------
//	Convert a C string to a Pascal string in place
//
//	On entry, ioStr should be a null-terminated C string
//	On exit, ioStr is a Pascal string with a length count in the first byte
//		Pascal string length is limited to 255 characters

StringPtr
LString::CToPStr(
	char*	ioStr)
{
	if (ioStr != NULL) {
		UInt8	len = LString::CStringLength(ioStr);

		memmove(ioStr + 1, ioStr, len);
		//::BlockMoveData(ioStr, ioStr + 1, len);

		ioStr[0] = (char) len;				// Store length count
	}

	return (StringPtr) ioStr;
}


// ---------------------------------------------------------------------------
//	 PToCStr
// ---------------------------------------------------------------------------
//	Convert a Pascal string to a C string in place
//
//	On entry, ioStr should be a Pascal string with a lenght count
//	On exit, ioStr is a null-terminated C string

char*
LString::PToCStr(
	StringPtr	ioStr)
{
	if (ioStr != NULL) {
		UInt8	len = ioStr[0];

		memmove(ioStr, ioStr + 1, len);
		//::BlockMoveData(ioStr + 1, ioStr, len);

		ioStr[len] = 0;						// Store null terminator
	}

	return (char*) ioStr;
}


// ---------------------------------------------------------------------------
//	 AppendPStr
// ---------------------------------------------------------------------------
//	Append two Pascal strings. The first string becomes the combination
//	of the first and second strings.
//
//	Returns a pointer to the resulting string
//
//	inDestSize specifies the maximum size of the base string
//		(INCLUDING the length byte). The default is sizeof Str255.
//
//	Call this function as follows:
//
//		AppendPStr(ioString, appendMe, sizeof(ioString));

StringPtr
LString::AppendPStr(
	Str255				ioBaseString,
	ConstStringPtr		inAppendString,
	SInt16				inDestSize)
{
								// Limit combined string to inDestSize chars
    SInt16	charsToCopy = inAppendString[0];
    if (ioBaseString[0] + charsToCopy > inDestSize - 1) {
    	charsToCopy = (SInt16) (inDestSize - 1 - ioBaseString[0]);
    }

								// Copy second to end of first string
	memmove(ioBaseString + ioBaseString[0] + 1, inAppendString + 1, charsToCopy);
    //::BlockMoveData(inAppendString + 1,  ioBaseString + ioBaseString[0] + 1, charsToCopy);
    							// Set length of combined string
    ioBaseString[0] += (UInt8) charsToCopy;

    return ioBaseString;
}

#pragma mark --- Protected Utilities ---

// ---------------------------------------------------------------------------
//		 LoadFromPtr
// ---------------------------------------------------------------------------
//	Fill in the String given a pointer and a byte count

void
LString::LoadFromPtr(
	const void*		inPtr,
	UInt8			inLength)
{
	if (inLength > mMaxBytes - 1) {		// mMaxBytes includes a length byte
										// Enforce maximum string length
		inLength = (UInt8) (mMaxBytes - 1);
	}

	memmove(mStringPtr + 1, inPtr, inLength);
	//::BlockMoveData(inPtr, mStringPtr + 1, inLength);
	mStringPtr[0] = inLength;
}
#pragma mark --- operator + ---

// ---------------------------------------------------------------------------
//		 operator+ (LString&, LString&)
// ---------------------------------------------------------------------------
//	Concatenate two String objects
//		leftString + rightString

LStr255
operator+(
	const LString&	inLeftString,
	const LString&	inRightString)
{
	LStr255	resultString = inLeftString;
	return resultString += inRightString;
}


// ---------------------------------------------------------------------------
//		 operator+ (LString&, ConstStringPtr)
// ---------------------------------------------------------------------------
//	Concatenate a String object and a string pointer
//		leftString + "Right";

LStr255
operator+(
	const LString&	inLeftString,
	ConstStringPtr	inRightStringPtr)
{
	LStr255	resultString = inLeftString;
	return resultString += inRightStringPtr;
}


// ---------------------------------------------------------------------------
//		 operator+ (ConstStringPtr, LString&)
// ---------------------------------------------------------------------------
//	Concatenate a string pointer and a String object
//		"Left" + rightString;

LStr255
operator+(
	ConstStringPtr	inLeftStringPtr,
	const LString&	inRightString)
{
	LStr255	resultString = inLeftStringPtr;
	return resultString += inRightString;
}


// ---------------------------------------------------------------------------
//		 operator+ (LString&, StringPtr)
// ---------------------------------------------------------------------------
//	Concatenate a String object and a string pointer
//		leftString + rightStringPtr;

LStr255
operator+(
	const LString&	inLeftString,
	StringPtr		inRightStringPtr)
{
	LStr255	resultString = inLeftString;
	return resultString += inRightStringPtr;
}


// ---------------------------------------------------------------------------
//		 operator+ (StringPtr, LString&)
// ---------------------------------------------------------------------------
//	Concatenate a string pointer and a String object
//		leftStringPtr + rightString;

LStr255
operator+(
	StringPtr		inLeftStringPtr,
	const LString&	inRightString)
{
	LStr255	resultString = inLeftStringPtr;
	return resultString += inRightString;
}


// ---------------------------------------------------------------------------
//		 operator+ (LString&, unsigned char)
// ---------------------------------------------------------------------------
//	Concatenate a String object and a character
//		leftString + 'a';

LStr255
operator+(
	const LString&	inLeftString,
	unsigned char	inRightChar)
{
	LStr255	resultString = inLeftString;
	return resultString += inRightChar;
}


// ---------------------------------------------------------------------------
//		 operator+ (unsigned char, LString&)
// ---------------------------------------------------------------------------
//	Concatenate a character and a String object
//		'a' + rightString;

LStr255
operator+(
	unsigned char	inLeftChar,
	const LString&	inRightString)
{
	LStr255	resultString = inLeftChar;
	return resultString += inRightString;
}


#pragma mark -
// ===========================================================================
//	 LStr255
// ===========================================================================
//	A String with a maximum of 255 characters

// ---------------------------------------------------------------------------
//		 LStr255
// ---------------------------------------------------------------------------
//	Default Constructor

LStr255::LStr255()

	: LString(sizeof(mString), mString)
{
	mString[0] = 0;
}


// ---------------------------------------------------------------------------
//		 LStr255 ( LStr255& )
// ---------------------------------------------------------------------------
//	Copy Constructor

LStr255::LStr255( const LStr255& inOriginal)
                : LString(sizeof(mString), mString)
{
    ConstStringPtr inOriginalAsConst = inOriginal;
    LoadFromStringPtr(inOriginalAsConst);
}


// ---------------------------------------------------------------------------
//		 LStr255 ( LString& )
// ---------------------------------------------------------------------------
//	Copy Constructor

LStr255::LStr255( const LString& inOriginal)
		: LString(sizeof(mString), mString)
{
    ConstStringPtr inOriginalAsConst = inOriginal;
    LoadFromStringPtr(inOriginalAsConst);
}


// ---------------------------------------------------------------------------
//		 LStr255 ( ConstStringPtr )
// ---------------------------------------------------------------------------
//	Constructor from a pointer to a string

LStr255::LStr255(
	ConstStringPtr	inStringPtr)
	
	: LString(sizeof(mString), mString)
{
	if (inStringPtr == NULL) {
		mString[0] = 0;
	} else {
		LoadFromStringPtr(inStringPtr);
	}
}


// ---------------------------------------------------------------------------
//		 LStr255 ( unsigned char )
// ---------------------------------------------------------------------------
//	Constructor from a single character

LStr255::LStr255(
	unsigned char	inChar)
	
	: LString(sizeof(mString), mString)
{
	mString[0] = 1;
	mString[1] = inChar;
}


// ---------------------------------------------------------------------------
//		 LStr255 ( char )
// ---------------------------------------------------------------------------
//	Constructor from a single character

LStr255::LStr255(
	char	inChar)
	
	: LString(sizeof(mString), mString)
{
	mString[0] = 1;
	mString[1] = (UInt8) inChar;
}


// ---------------------------------------------------------------------------
//		 LStr255 ( const char* )
// ---------------------------------------------------------------------------
//	Constructor from a C string (null terminated)

LStr255::LStr255(
	const char*	inCString)
	
	: LString(sizeof(mString), mString)
{
	LoadFromPtr(inCString,  CStringLength(inCString));
}


// ---------------------------------------------------------------------------
//		 LStr255 ( const void*, UInt8 )
// ---------------------------------------------------------------------------
//	Constructor from a pointer and length

LStr255::LStr255(
	const void*		inPtr,
	UInt8			inLength)
		
	: LString(sizeof(mString), mString)
{
	LoadFromPtr(inPtr, inLength);
}

// ---------------------------------------------------------------------------
//		 LStr255 ( SInt32 )
// ---------------------------------------------------------------------------
//	Constructor from a long integer

LStr255::LStr255(
	SInt32	inNumber)
	
	: LString(sizeof(mString), mString)
{
	Assign(inNumber);
}


// ---------------------------------------------------------------------------
//		 LStr255 ( SInt16 )
// ---------------------------------------------------------------------------
//	Constructor from a long integer

LStr255::LStr255(
	SInt16	inNumber)
	
	: LString(sizeof(mString), mString)
{
	Assign(inNumber);
}

#pragma mark -
// ===========================================================================
//	 LStringRef
// ===========================================================================
//	Class does not contain storage for the string. The StringPtr points
//	to a Pascal string allocated elsewhere

// ---------------------------------------------------------------------------
//	 LStringRef							Default Constructor
// ---------------------------------------------------------------------------

LStringRef::LStringRef(
	SInt16		inMaxBytes,
	StringPtr	inStringPtr)	// Must point to buffer of inMaxBytes bytes
	
	: LString(inMaxBytes, inStringPtr)
{
}


// ---------------------------------------------------------------------------
//	 LStringRef							Copy Constructor
// ---------------------------------------------------------------------------
//	Copy points to the same string as the original

LStringRef::LStringRef(
	const LStringRef&	inStringRef)
	
	: LString((SInt16) inStringRef.mMaxBytes, inStringRef.mStringPtr)
{
	mCompareFunc = inStringRef.mCompareFunc;
}


// ---------------------------------------------------------------------------
//	 LStringRef							Assignment Operator
// ---------------------------------------------------------------------------
//	Object points to the same string as the rhs operand

LStringRef&
LStringRef::operator = (
	const LStringRef&	inStringRef)
{
	mStringPtr	 = inStringRef.mStringPtr;
	mCompareFunc = inStringRef.mCompareFunc;
	mMaxBytes	 = inStringRef.mMaxBytes;

	return *this;
}


// ---------------------------------------------------------------------------
//	 SetStringRef
// ---------------------------------------------------------------------------
//	Specify the string referenced by this object

void
LStringRef::SetStringRef(
	UInt16		inMaxBytes,
	StringPtr	inStringPtr)	// Must point to buffer of inMaxBytes bytes
{
	mMaxBytes	= inMaxBytes;
	mStringPtr	= inStringPtr;
}
